package com.chelvc.cloud.admin.controller;

import javax.validation.Valid;
import javax.validation.constraints.Min;

import com.chelvc.cloud.uc.api.dto.ClientDTO;
import com.chelvc.cloud.uc.api.param.ClientModifyParam;
import com.chelvc.cloud.uc.api.param.ClientPagingParam;
import com.chelvc.cloud.uc.api.service.ClientService;
import com.chelvc.framework.base.annotation.UnifiedResponseBody;
import com.chelvc.framework.base.model.Pagination;
import com.chelvc.framework.base.util.ErrorUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 客户端接口
 *
 * @author Woody
 * @date 2021/10/20
 */
@Validated
@RestController
@UnifiedResponseBody
@PreAuthorize("isBusiness('EMPLOYEE')")
public class ClientController {
    @DubboReference
    private ClientService clientService;

    /**
     * 新增客户端
     *
     * @param param 新增参数
     * @return 客户端主键
     */
    @PostMapping("/client")
    public Long addClient(@RequestBody @Valid ClientModifyParam param) {
        return this.clientService.addClient(param);
    }

    /**
     * 修改客户端
     *
     * @param id    客户端主键
     * @param param 修改参数
     */
    @PutMapping("/client/{id}")
    public void updateClient(@PathVariable("id") @Min(value = 1, message = "客户端主键不能小于1") Long id,
                             @RequestBody @Valid ClientModifyParam param) {
        this.clientService.updateClient(id, param);
    }

    /**
     * 获取客户端信息
     *
     * @param id 客户端主键
     * @return 客户端信息
     */
    @GetMapping("/client/{id}")
    public ClientDTO getClient(@PathVariable("id") @Min(value = 1, message = "客户端主键不能小于1") Long id) {
        ClientDTO client = this.clientService.getClient(id);
        ErrorUtils.requireResource(client, "客户端不存在");
        return client;
    }

    /**
     * 查询客户端分页
     *
     * @param param 查询参数
     * @return 客户端分页信息
     */
    @GetMapping("/client/paging")
    public Pagination<ClientDTO> getClientPaging(@Valid ClientPagingParam param) {
        return this.clientService.getClientPaging(param);
    }
}
